(function (Drupal) {
  Drupal.behaviors.magicCodeVerifyInput = {
    attach(context) {
      once("magicCodeVerifyInput", ".verify-code-input", context).forEach(
        /**
         * @param {HTMLElement} element
         */
        (element) => {
          const inputs = element.querySelectorAll("input");

          /**
           * Submit the form.
           */
          function submitForm() {
            const form = element.closest("form");
            if (form) {
              form.dispatchEvent(new Event("submit"));
              form.submit();
            }
          }

          /**
           * Map the code to each individual input.
           */
          function setCode(code, autosubmit = false) {
            code
              .replace("-", "")
              .split("")
              .forEach((char, index) => {
                inputs[index].value = char;
              });

            if (autosubmit) {
              submitForm();
            }
          }

          inputs.forEach((input, index) => {
            // Handle input.
            input.addEventListener("input", () => {
              if (input.value === "") {
                return;
              }

              // If input, focus next one.
              if (index < 5) {
                inputs[index + 1]?.focus();
              } else {
                const form = element.closest("form");
                const data = new FormData(form);

                const parts = [
                  data.get("code_input_1"),
                  data.get("code_input_2"),
                  data.get("code_input_3"),
                  "-",
                  data.get("code_input_4"),
                  data.get("code_input_5"),
                  data.get("code_input_6"),
                ];

                const code = parts.join("");

                // Autosubmit if the code is valid.
                const regex = /^[a-zA-Z0-9]{3}-[a-zA-Z0-9]{3}$/;
                if (regex.test(code)) {
                  submitForm();
                }
              }
            });

            // Handle keydown (backspace).
            input.addEventListener("keydown", (event) => {
              if (input.value !== "" || event.key !== "Backspace") {
                return;
              }

              // Focus previous one.
              if (index > 0) {
                inputs[index - 1]?.focus();
              }
            });

            // Handle focus.
            input.addEventListener("focus", () => {
              if (input.value === "") {
                return;
              }

              input.select();
            });

            // Handle paste.
            input.addEventListener("paste", (event) => {
              event.preventDefault();

              const text = event.clipboardData.getData("text").trim();

              // Validate input.
              if (text.length !== 7 || text[3] !== "-") {
                return;
              }

              setCode(text, true);
            });
          });

          /**
           * Prefill from query params.
           */
          const code = new URLSearchParams(location.search).get("code");
          if (code) {
            setCode(code);
          }
        }
      );
    },
  };
})(Drupal);
